package com.webchat.service.lottery;

import com.webchat.common.constants.LotteryConstants;
import com.webchat.common.exception.BusinessException;
import com.webchat.domain.vo.request.lottery.LotteryActivityStatusUpdateVO;
import com.webchat.domain.vo.response.lottery.LotteryActivityVO;
import com.webchat.service.queue.LotteryOrderQueue;
import com.webchat.service.queue.dto.LotteryOrderQueueDTO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.concurrent.ThreadLocalRandom;

/**
 * @Author 程序员七七
 * @webSite https://www.coderutil.com
 * @Date 2022/11/16 23:40
 * @description
 */
@Slf4j
@Service
public class LotteryService {

    @Autowired
    private LotteryOrderQueue lotteryOrderQueue;
    @Autowired
    private LotteryOrderService lotteryOrderService;
    @Autowired
    private LotteryActivityService lotteryActivityService;
    @Autowired
    private LotteryItemService lotteryItemService;
    @Autowired
    private LotteryCacheService lotteryCacheService;

    /**
     * 幸运抽奖
     *
     * @param lotteryActivityId 抽奖活动id
     * @param userId            当前登录用户id
     * @return 中奖商品id
     */
    public Long luckDraw(String lotteryActivityId, String userId) {

        /**
         * 1、抽奖
         */
        Long lotteryItemId = this.doLotteryDraw(lotteryActivityId);

        /**
         * 2、处理中奖订单
         */
        this.handleLotteryOrder(userId, lotteryActivityId, lotteryItemId);

        return lotteryItemId;
    }

    /**
     * 多次抽奖
     *
     * @param lotteryActivityId
     * @param userId
     * @param times
     * @return
     */
    public synchronized List<Long> luckDraw(String lotteryActivityId, String userId, int times) {
        List<Long> result = new Vector<>();
        for (int i = 0; i < times; i++) {
            Long lotteryItemId = this.doLotteryDraw(lotteryActivityId);
            this.handleLotteryOrder(userId, lotteryActivityId, lotteryItemId);
            result.add(lotteryItemId);
        }
        return result;
    }

    /**
     * 最新一期活动抽奖
     * @param userId
     * @return
     */
    public Long luckDrawLastActivity(String userId) {
        String activityId = lotteryCacheService.getLastLotteryActivityIdFromCache();
        Assert.isTrue(StringUtils.isNotBlank(activityId), "未查询到最新一期的抽奖活动");
        return this.luckDraw(activityId, userId);
    }

    /**
     * 抽奖，返回抽中奖品的id
     *
     * @param lotteryActivityId
     * @return
     */
    private Long doLotteryDraw(String lotteryActivityId) {

        /**
         * 1、抽奖
         */
        return this.executeLotteryDraw(lotteryActivityId);
    }

    /**
     * 执行抽奖
     *
     * @param lotteryActivityId
     * @return
     */
    private synchronized Long executeLotteryDraw(String lotteryActivityId) {
        /**
         * 获取奖品库存
         */
        Map<Long, Integer> stockMap = lotteryCacheService.getLotteryItemStockFromCache(lotteryActivityId);
        /**
         * 构造奖品池
         */
        List<Long> itemPool = new ArrayList<>();
        for (Map.Entry<Long, Integer> entry : stockMap.entrySet()) {
            Long itemId = entry.getKey();
            int stock = entry.getValue();
            if (stock == 0) {
                continue;
            }
            for (int i = 0; i < stock; i++) {
                itemPool.add(itemId);
            }
        }

        /**
         * 奖品已抽完，自动结束活动
         */
        if (itemPool.size() == 0) {
            LotteryActivityStatusUpdateVO lotteryActivityStatusUpdate = new LotteryActivityStatusUpdateVO();
            lotteryActivityStatusUpdate.setActivityId(lotteryActivityId);
            lotteryActivityStatusUpdate.setStatus(LotteryConstants.LotteryActivityStatus.CLOSED.getStatus());
            lotteryActivityService.updateLotteryActivityStatus(lotteryActivityStatusUpdate, LotteryConstants.SYSTEM);
            throw new BusinessException("本期奖品抽完。活动已结束！");
        }

        /**
         * 随机抽奖
         */
        int luckIndex = this.randomLotteryIndex(itemPool.size());
        Long luckItemId = itemPool.get(luckIndex);
        /**
         * 扣减库存
         */
        lotteryCacheService.deductItemStock(lotteryActivityId, luckItemId);
        return luckItemId;
    }

    /**
     * 生成抽奖索引
     *
     * @param maxIndex
     * @return
     */
    public int randomLotteryIndex(int maxIndex) {
        ThreadLocalRandom threadLocalRandom = ThreadLocalRandom.current();
        return threadLocalRandom.nextInt(0, maxIndex);
    }

    /**
     * 校验是否满足抽奖条件
     *
     * @param lotteryActivityId
     * @param userId
     * @return 满足抽奖条件返回抽奖需要消耗的元气值个数
     */
    private int validateLotteryDrawCondition(String lotteryActivityId, String userId, int times) {

        // 不需要检验元气
        return Integer.MAX_VALUE;
    }

    /**
     * 校验抽奖活动
     */
    private LotteryActivityVO validateLotteryActivity(String lotteryActivityId) {
        LotteryActivityVO lotteryActivity = lotteryCacheService.getLotteryActivityDetailFromCache(lotteryActivityId);
        Assert.isTrue(lotteryActivity != null, "活动不存在！");
        // 校验活动是否进行中
        lotteryActivity.validateRunningStatus();
        // 校验活动积分配置
        lotteryActivity.validateIntegral();
        return lotteryActivity;
    }

    /**
     * 中奖结果加入队列，排队处理
     *
     * @param lotteryActivityId
     * @param userId
     * @param itemId
     */
    private void handleLotteryOrder(String userId, String lotteryActivityId, Long itemId) {
        LotteryOrderQueueDTO lotteryOrderQueueDTO = new LotteryOrderQueueDTO();
        lotteryOrderQueueDTO.setActivityId(lotteryActivityId);
        lotteryOrderQueueDTO.setItemId(itemId);
        lotteryOrderQueueDTO.setUserId(userId);
        // 消息队列处理订单
        lotteryOrderQueue.submit(lotteryOrderQueueDTO);
    }
}
